@@ -39,7 +39,6 @@
 #include "gdl-dock-item-grip.h"
 #include "gdl-dock-item-button-image.h"
 #include "gdl-switcher.h"
-#include "gdl-tools.h"
 
 #define ALIGN_BORDER 5
 #define DRAG_HANDLE_SIZE 10
@@ -58,8 +57,7 @@ struct _GdlDockItemGripPrivate {
     gboolean    handle_shown;
 };
  
-GDL_CLASS_BOILERPLATE (GdlDockItemGrip, gdl_dock_item_grip,
-                       GtkContainer, GTK_TYPE_CONTAINER);
+G_DEFINE_TYPE (GdlDockItemGrip, gdl_dock_item_grip, GTK_TYPE_CONTAINER);
                        
 GtkWidget*
 gdl_dock_item_create_label_widget(GdlDockItemGrip *grip)
@@ -102,35 +100,26 @@ gdl_dock_item_create_label_widget(GdlDockItemGrip *grip)
     return GTK_WIDGET(label_box);
 }
 
-static gint
-gdl_dock_item_grip_expose (GtkWidget      *widget,
-                           GdkEventExpose *event)
+static gboolean
+gdl_dock_item_grip_draw (GtkWidget      *widget,
+                         cairo_t *cr)
 {
     GdlDockItemGrip *grip;
-    GdkRectangle handle_area;
-    GdkRectangle expose_area;
+    GtkAllocation allocation;
+    cairo_rectangle_int_t handle_area;
+    cairo_rectangle_int_t expose_area;
 
     grip = GDL_DOCK_ITEM_GRIP (widget);
-    
-    if(grip->_priv->handle_shown) {
-        
-        if (gtk_widget_get_direction (widget) != GTK_TEXT_DIR_RTL) {
-            handle_area.x = widget->allocation.x;
-            handle_area.y = widget->allocation.y;
-            handle_area.width = DRAG_HANDLE_SIZE;
-            handle_area.height = widget->allocation.height;
-        } else {
-            handle_area.x = widget->allocation.x + widget->allocation.width
-                - DRAG_HANDLE_SIZE;
-            handle_area.y = widget->allocation.y;
-            handle_area.width = DRAG_HANDLE_SIZE;
-            handle_area.height = widget->allocation.height;  
-        }
 
-        if (gdk_rectangle_intersect (&handle_area, &event->area, &expose_area)) {
-
-            gtk_paint_handle (widget->style, widget->window, widget->state,
-                              GTK_SHADOW_NONE, &expose_area, widget,
+    if (grip->_priv->handle_shown) 
+    {
+        if (gtk_cairo_should_draw_window (cr, gtk_widget_get_window (widget)))
+        {
+            gtk_paint_handle (gtk_widget_get_style (widget),
+                              cr,
+                              gtk_widget_get_state (widget),
+                              GTK_SHADOW_NONE, 
+                              widget,
                               "handlebox", handle_area.x, handle_area.y,
                               handle_area.width, handle_area.height,
                               GTK_ORIENTATION_VERTICAL);
@@ -139,7 +128,7 @@ gdl_dock_item_grip_expose (GtkWidget      *widget,
         
     }
 
-    return GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
+    return GTK_WIDGET_CLASS (gdl_dock_item_grip_parent_class)->draw (widget, cr);
 }
 
 static void
@@ -183,7 +172,7 @@ gdl_dock_item_grip_item_notify (GObject    *master,
 }
 
 static void
-gdl_dock_item_grip_destroy (GtkObject *object)
+gdl_dock_item_grip_destroy (GtkWidget *object)
 {
     GdlDockItemGrip *grip = GDL_DOCK_ITEM_GRIP (object);
         
@@ -205,7 +194,7 @@ gdl_dock_item_grip_destroy (GtkObject *object)
         g_free (priv);
     }
     
-    GDL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
+    GTK_WIDGET_CLASS (gdl_dock_item_grip_parent_class)->destroy (object);
 }
 
 static void
@@ -255,6 +244,44 @@ gdl_dock_item_grip_close_clicked (GtkWidget       *widget,
     gdl_dock_item_hide_item (grip->item);
 }
 
+static void
+gdl_dock_item_grip_fix_iconify_button (GdlDockItemGrip *grip)
+{
+    GtkWidget *iconify_button = grip->_priv->iconify_button;
+    GdkWindow *window = NULL;
+    GdkEvent  *event = NULL;
+
+    GdkModifierType modifiers;
+    gint x = 0, y = 0;
+    gboolean ev_ret;
+
+    g_return_if_fail (gtk_widget_get_realized (iconify_button));
+
+    window = gtk_widget_get_parent_window (iconify_button);
+    event = gdk_event_new (GDK_LEAVE_NOTIFY);
+
+    g_assert (GDK_IS_WINDOW (window));
+    gdk_window_get_pointer (window, &x, &y, &modifiers);
+
+    event->crossing.window = g_object_ref (window);
+    event->crossing.send_event = FALSE;
+    event->crossing.subwindow = g_object_ref (window);
+    event->crossing.time = GDK_CURRENT_TIME;
+    event->crossing.x = x;
+    event->crossing.y = y;
+    event->crossing.x_root = event->crossing.y_root = 0;
+    event->crossing.mode = GDK_CROSSING_STATE_CHANGED;
+    event->crossing.detail = GDK_NOTIFY_NONLINEAR;
+    event->crossing.focus = FALSE;
+    event->crossing.state = modifiers;
+
+    //GTK_BUTTON (iconify_button)->in_button = FALSE;
+    g_signal_emit_by_name (iconify_button, "leave-notify-event",
+                           event, &ev_ret, 0);
+
+    gdk_event_free (event);
+}
+
 static void
 gdl_dock_item_grip_iconify_clicked (GtkWidget       *widget,
                                     GdlDockItemGrip *grip)
@@ -263,6 +290,9 @@ gdl_dock_item_grip_iconify_clicked (GtkWidget       *widget,
     
     g_return_if_fail (grip->item != NULL);
 
+    /* Workaround to unhighlight the iconify button. */
+    gdl_dock_item_grip_fix_iconify_button (grip);
+
     parent = gtk_widget_get_parent (GTK_WIDGET (grip->item));
     if (GDL_IS_SWITCHER (parent))
     {
@@ -286,18 +316,14 @@ gdl_dock_item_grip_iconify_clicked (GtkWidget       *widget,
     {
         gdl_dock_item_iconify_item (grip->item);
     }
-    
-    /* Workaround to unhighlight the iconify button. */
-    GTK_BUTTON (grip->_priv->iconify_button)->in_button = FALSE;
-    gtk_button_leave (GTK_BUTTON (grip->_priv->iconify_button));
 }
   
 static void
-gdl_dock_item_grip_instance_init (GdlDockItemGrip *grip)
+gdl_dock_item_grip_init (GdlDockItemGrip *grip)
 {
     GtkWidget *image;
 
-    GTK_WIDGET_SET_FLAGS (grip, GTK_NO_WINDOW);
+    gtk_widget_set_has_window (GTK_WIDGET (grip), FALSE);
     
     grip->_priv = g_new0 (GdlDockItemGripPrivate, 1);
     grip->_priv->label = NULL;
@@ -307,8 +333,8 @@ gdl_dock_item_grip_instance_init (GdlDockItemGrip *grip)
     gtk_widget_push_composite_child ();
     grip->_priv->close_button = gtk_button_new ();
     gtk_widget_pop_composite_child ();
-    
-    GTK_WIDGET_UNSET_FLAGS (grip->_priv->close_button, GTK_CAN_FOCUS);
+
+    gtk_widget_set_can_focus (grip->_priv->close_button, FALSE);
     gtk_widget_set_parent (grip->_priv->close_button, GTK_WIDGET (grip));
     gtk_button_set_relief (GTK_BUTTON (grip->_priv->close_button), GTK_RELIEF_NONE);
     gtk_widget_show (grip->_priv->close_button);
@@ -324,8 +350,8 @@ gdl_dock_item_grip_instance_init (GdlDockItemGrip *grip)
     gtk_widget_push_composite_child ();
     grip->_priv->iconify_button = gtk_button_new ();
     gtk_widget_pop_composite_child ();
-    
-    GTK_WIDGET_UNSET_FLAGS (grip->_priv->iconify_button, GTK_CAN_FOCUS);
+
+    gtk_widget_set_can_focus (grip->_priv->iconify_button, FALSE);
     gtk_widget_set_parent (grip->_priv->iconify_button, GTK_WIDGET (grip));
     gtk_button_set_relief (GTK_BUTTON (grip->_priv->iconify_button), GTK_RELIEF_NONE);
     gtk_widget_show (grip->_priv->iconify_button);
@@ -349,20 +375,23 @@ gdl_dock_item_grip_realize (GtkWidget *widget)
 {
     GdlDockItemGrip *grip = GDL_DOCK_ITEM_GRIP (widget);
 
-    GTK_WIDGET_CLASS (parent_class)->realize (widget);
+    GTK_WIDGET_CLASS (gdl_dock_item_grip_parent_class)->realize (widget);
 
     g_return_if_fail (grip->_priv != NULL);
     
     if (!grip->title_window) {
+        GtkAllocation  allocation;
         GdkWindowAttr  attributes;
         GdkCursor     *cursor;
 
         g_return_if_fail (grip->_priv->label != NULL);
 
-        attributes.x           = grip->_priv->label->allocation.x;
-        attributes.y           = grip->_priv->label->allocation.y;
-        attributes.width       = grip->_priv->label->allocation.width;
-        attributes.height      = grip->_priv->label->allocation.height;
+        gtk_widget_get_allocation (grip->_priv->label, &allocation);
+
+        attributes.x           = allocation.x;
+        attributes.y           = allocation.y;
+        attributes.width       = allocation.width;
+        attributes.height      = allocation.height;
         attributes.window_type = GDK_WINDOW_CHILD;
         attributes.wclass      = GDK_INPUT_OUTPUT;
         attributes.event_mask  = GDK_ALL_EVENTS_MASK;
@@ -373,11 +402,11 @@ gdl_dock_item_grip_realize (GtkWidget *widget)
         gdk_window_set_user_data (grip->title_window, grip);
 
         /* Unref the ref from parent realize for NO_WINDOW */
-        g_object_unref (widget->window);
+        g_object_unref (gtk_widget_get_window (widget));
 
         /* Need to ref widget->window, because parent unrealize unrefs it */
-        widget->window = g_object_ref (grip->title_window);
-        GTK_WIDGET_UNSET_FLAGS(widget, GTK_NO_WINDOW);
+        gtk_widget_set_window (widget, g_object_ref (grip->title_window));
+        gtk_widget_set_has_window (widget, TRUE);
 
         /* Unset the background so as to make the colour match the parent window */
         gtk_widget_modify_bg(widget, GTK_STATE_NORMAL, NULL);
@@ -400,13 +429,13 @@ gdl_dock_item_grip_unrealize (GtkWidget *widget)
     GdlDockItemGrip *grip = GDL_DOCK_ITEM_GRIP (widget);
 
     if (grip->title_window) {
-        GTK_WIDGET_SET_FLAGS(widget, GTK_NO_WINDOW);
+        gtk_widget_set_has_window (widget, FALSE);
         gdk_window_set_user_data (grip->title_window, NULL);
         gdk_window_destroy (grip->title_window);
         grip->title_window = NULL;
     }
 
-    GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+    GTK_WIDGET_CLASS (gdl_dock_item_grip_parent_class)->unrealize (widget);
 }
 
 static void
@@ -414,7 +443,7 @@ gdl_dock_item_grip_map (GtkWidget *widget)
 {
     GdlDockItemGrip *grip = GDL_DOCK_ITEM_GRIP (widget);
 
-    GTK_WIDGET_CLASS (parent_class)->map (widget);
+    GTK_WIDGET_CLASS (gdl_dock_item_grip_parent_class)->map (widget);
 
     if (grip->title_window)
         gdk_window_show (grip->title_window);
@@ -428,47 +457,71 @@ gdl_dock_item_grip_unmap (GtkWidget *widget)
     if (grip->title_window)
         gdk_window_hide (grip->title_window);
 
-    GTK_WIDGET_CLASS (parent_class)->unmap (widget);
+    GTK_WIDGET_CLASS (gdl_dock_item_grip_parent_class)->unmap (widget);
 }
 
 static void
-gdl_dock_item_grip_size_request (GtkWidget      *widget,
-                                 GtkRequisition *requisition)
+gdl_dock_item_grip_get_preferred_width (GtkWidget *widget,
+                                        gint      *minimum,
+                                        gint      *natural)
 {
-    GtkRequisition   child_requisition;
-    GtkContainer    *container;
     GdlDockItemGrip *grip;
-    gint             layout_height = 0;
+    gint child_min, child_nat;
 
     g_return_if_fail (GDL_IS_DOCK_ITEM_GRIP (widget));
-    g_return_if_fail (requisition != NULL);
 
-    container = GTK_CONTAINER (widget);
     grip = GDL_DOCK_ITEM_GRIP (widget);
-    
-    requisition->width = container->border_width * 2/* + ALIGN_BORDER*/;
-    requisition->height = container->border_width * 2;
 
-    if(grip->_priv->handle_shown)
-        requisition->width += DRAG_HANDLE_SIZE;
+    *minimum = *natural = 0;
+
+    if(grip->_priv->handle_shown) {
+        *minimum += DRAG_HANDLE_SIZE;
+        *natural += DRAG_HANDLE_SIZE;
+    }
 
-    gtk_widget_size_request (grip->_priv->close_button, &child_requisition);
-    layout_height = MAX (layout_height, child_requisition.height);
     if (gtk_widget_get_visible (grip->_priv->close_button)) {
-       requisition->width += child_requisition.width;
+       gtk_widget_get_preferred_width (grip->_priv->close_button, &child_min, &child_nat);
+       *minimum += child_min;
+       *natural += child_nat;
     }
-    
-    gtk_widget_size_request (grip->_priv->iconify_button, &child_requisition);
-    layout_height = MAX (layout_height, child_requisition.height);
+
     if (gtk_widget_get_visible (grip->_priv->iconify_button)) {
-        requisition->width += child_requisition.width;
+        gtk_widget_get_preferred_width (grip->_priv->iconify_button, &child_min, &child_nat);
+       *minimum += child_min;
+       *natural += child_nat;
     }
-   
-    gtk_widget_size_request (grip->_priv->label, &child_requisition);
-    requisition->width += child_requisition.width;
-    layout_height = MAX (layout_height, child_requisition.height);
-    
-    requisition->height += layout_height;
+
+    gtk_widget_get_preferred_width (grip->_priv->label, &child_min, &child_nat);
+    *minimum += child_min;
+    *natural += child_nat;
+}
+
+static void
+gdl_dock_item_grip_get_preferred_height (GtkWidget *widget,
+                                         gint      *minimum,
+                                         gint      *natural)
+{
+    GdlDockItemGrip *grip;
+    gint child_min, child_nat;
+    gint layout_height = 0;
+
+    g_return_if_fail (GDL_IS_DOCK_ITEM_GRIP (widget));
+
+    grip = GDL_DOCK_ITEM_GRIP (widget);
+
+    *minimum = *natural = 0;
+
+    gtk_widget_get_preferred_height (grip->_priv->close_button, &child_min, &child_nat);
+    *minimum = MAX (*minimum, child_min);
+    *natural = MAX (*natural, child_nat);
+
+    gtk_widget_get_preferred_height (grip->_priv->iconify_button, &child_min, &child_nat);
+    *minimum = MAX (*minimum, child_min);
+    *natural = MAX (*natural, child_nat);
+
+    gtk_widget_get_preferred_height (grip->_priv->label, &child_min, &child_nat);
+    *minimum = MAX (*minimum, child_min);
+    *natural = MAX (*natural, child_nat);
 }
 
 static void
@@ -476,7 +529,6 @@ gdl_dock_item_grip_size_allocate (GtkWidget     *widget,
                                   GtkAllocation *allocation)
 {
     GdlDockItemGrip *grip;
-    GtkContainer    *container;
     GtkRequisition   close_requisition = { 0, };
     GtkRequisition   iconify_requisition = { 0, };
     GtkAllocation    child_allocation;
@@ -486,28 +538,26 @@ gdl_dock_item_grip_size_allocate (GtkWidget     *widget,
     g_return_if_fail (allocation != NULL);
   
     grip = GDL_DOCK_ITEM_GRIP (widget);
-    container = GTK_CONTAINER (widget);
 
-    GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
+    GTK_WIDGET_CLASS (gdl_dock_item_grip_parent_class)->size_allocate (widget, allocation);
 
-    gtk_widget_size_request (grip->_priv->close_button,
-        &close_requisition);
-    gtk_widget_size_request (grip->_priv->iconify_button,
-        &iconify_requisition);
+    gtk_widget_get_preferred_size (grip->_priv->close_button,
+        &close_requisition, NULL);
+    gtk_widget_get_preferred_size (grip->_priv->iconify_button,
+        &iconify_requisition, NULL);
     
     /* Calculate the Minimum Width where buttons will fit */
-    int min_width = close_requisition.width + iconify_requisition.width
-        + container->border_width * 2;
+    int min_width = close_requisition.width + iconify_requisition.width;
     if(grip->_priv->handle_shown)
       min_width += DRAG_HANDLE_SIZE;
     const gboolean space_for_buttons = (allocation->width >= min_width);
         
     /* Set up the rolling child_allocation rectangle */
     if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
-        child_allocation.x = container->border_width/* + ALIGN_BORDER*/;
+        child_allocation.x = 0;
     else
-        child_allocation.x = allocation->width - container->border_width;
-    child_allocation.y = container->border_width;
+        child_allocation.x = allocation->width;
+    child_allocation.y = 0;
 
     /* Layout Close Button */
     if (gtk_widget_get_visible (grip->_priv->close_button)) {
@@ -550,7 +600,7 @@ gdl_dock_item_grip_size_allocate (GtkWidget     *widget,
     /* Layout the Grip Handle*/
     if (gtk_widget_get_direction (widget) != GTK_TEXT_DIR_RTL) {
         child_allocation.width = child_allocation.x;
-        child_allocation.x = container->border_width/* + ALIGN_BORDER*/;
+        child_allocation.x = 0;
         
         if(grip->_priv->handle_shown) {
             child_allocation.x += DRAG_HANDLE_SIZE;
@@ -568,8 +618,8 @@ gdl_dock_item_grip_size_allocate (GtkWidget     *widget,
     if(child_allocation.width < 0)
       child_allocation.width = 0;
     
-    child_allocation.y = container->border_width;
-    child_allocation.height = allocation->height - container->border_width * 2;
+    child_allocation.y = 0;
+    child_allocation.height = allocation->height;
     if(grip->_priv->label) {
       gtk_widget_size_allocate (grip->_priv->label, &child_allocation);
     }
@@ -630,32 +680,31 @@ static void
 gdl_dock_item_grip_class_init (GdlDockItemGripClass *klass)
 {
     GObjectClass *gobject_class;
-    GtkObjectClass *gtk_object_class;
     GtkWidgetClass *widget_class;
     GtkContainerClass *container_class;
 
-    parent_class = g_type_class_peek_parent (klass);
     gobject_class = G_OBJECT_CLASS (klass);
-    gtk_object_class = GTK_OBJECT_CLASS (klass);
     widget_class = GTK_WIDGET_CLASS (klass);
     container_class = GTK_CONTAINER_CLASS (klass);
 
     gobject_class->set_property = gdl_dock_item_grip_set_property;
 
-    gtk_object_class->destroy = gdl_dock_item_grip_destroy;
+    widget_class->destroy = gdl_dock_item_grip_destroy;
 
-    widget_class->expose_event = gdl_dock_item_grip_expose;
+    widget_class->draw = gdl_dock_item_grip_draw;
     widget_class->realize = gdl_dock_item_grip_realize;
     widget_class->unrealize = gdl_dock_item_grip_unrealize;
     widget_class->map = gdl_dock_item_grip_map;
     widget_class->unmap = gdl_dock_item_grip_unmap;
-    widget_class->size_request = gdl_dock_item_grip_size_request;
+    widget_class->get_preferred_width = gdl_dock_item_grip_get_preferred_width;
+    widget_class->get_preferred_height = gdl_dock_item_grip_get_preferred_height;
     widget_class->size_allocate = gdl_dock_item_grip_size_allocate;
 
     container_class->add = gdl_dock_item_grip_add;
     container_class->remove = gdl_dock_item_grip_remove;
     container_class->forall = gdl_dock_item_grip_forall;
     container_class->child_type = gdl_dock_item_grip_child_type;
+    gtk_container_class_handle_border_width (container_class);
 
     g_object_class_install_property (
         gobject_class, PROP_ITEM,
@@ -717,7 +766,7 @@ gdl_dock_item_grip_set_label (GdlDockItemGrip *grip,
 }
 /**
  * gdl_dock_item_grip_hide_handle:
- * @item: The dock item grip to hide the handle of.
+ * @grip: The dock item grip to hide the handle of.
  * 
  * This function hides the dock item's grip widget handle hatching.
  **/
