@@ -29,7 +29,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "gdl-tools.h"
 #include "gdl-dock-object.h"
 #include "gdl-dock-master.h"
 #include "libgdltypebuiltins.h"
@@ -46,7 +45,6 @@
 /* ----- Private prototypes ----- */
 
 static void     gdl_dock_object_class_init         (GdlDockObjectClass *klass);
-static void     gdl_dock_object_instance_init      (GdlDockObject      *object);
 
 static void     gdl_dock_object_set_property       (GObject            *g_object,
                                                     guint               prop_id,
@@ -58,7 +56,7 @@ static void     gdl_dock_object_get_property       (GObject            *g_object
                                                     GParamSpec         *pspec);
 static void     gdl_dock_object_finalize           (GObject            *g_object);
 
-static void     gdl_dock_object_destroy            (GtkObject          *gtk_object);
+static void     gdl_dock_object_destroy            (GtkWidget          *dock_object);
 
 static void     gdl_dock_object_show               (GtkWidget          *widget);
 static void     gdl_dock_object_hide               (GtkWidget          *widget);
@@ -95,18 +93,16 @@ static guint gdl_dock_object_signals [LAST_SIGNAL] = { 0 };
 
 /* ----- Private interface ----- */
 
-GDL_CLASS_BOILERPLATE (GdlDockObject, gdl_dock_object, GtkContainer, GTK_TYPE_CONTAINER);
+G_DEFINE_TYPE (GdlDockObject, gdl_dock_object, GTK_TYPE_CONTAINER);
 
 static void
 gdl_dock_object_class_init (GdlDockObjectClass *klass)
 {
     GObjectClass      *g_object_class;
-    GtkObjectClass    *object_class;
     GtkWidgetClass    *widget_class;
     GtkContainerClass *container_class;
 
     g_object_class = G_OBJECT_CLASS (klass);
-    object_class = GTK_OBJECT_CLASS (klass);
     widget_class = GTK_WIDGET_CLASS (klass);
     container_class = GTK_CONTAINER_CLASS (klass);
 
@@ -143,7 +139,7 @@ gdl_dock_object_class_init (GdlDockObjectClass *klass)
                              GDL_TYPE_DOCK_MASTER,
                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
     
-    object_class->destroy = gdl_dock_object_destroy;
+    widget_class->destroy = gdl_dock_object_destroy;
     
     widget_class->show = gdl_dock_object_show;
     widget_class->hide = gdl_dock_object_hide;
@@ -186,7 +182,7 @@ gdl_dock_object_class_init (GdlDockObjectClass *klass)
 }
 
 static void
-gdl_dock_object_instance_init (GdlDockObject *object)
+gdl_dock_object_init (GdlDockObject *object)
 {
     object->flags = GDL_DOCK_AUTOMATIC;
     object->freeze_count = 0;
@@ -268,7 +264,7 @@ gdl_dock_object_finalize (GObject *g_object)
     g_free (object->stock_id);
     object->stock_id = NULL;
 
-    GDL_CALL_PARENT (G_OBJECT_CLASS, finalize, (g_object));
+    G_OBJECT_CLASS (gdl_dock_object_parent_class)->finalize (g_object);
 }
 
 static void
@@ -279,13 +275,13 @@ gdl_dock_object_foreach_detach (GdlDockObject *object,
 }
 
 static void
-gdl_dock_object_destroy (GtkObject *gtk_object)
+gdl_dock_object_destroy (GtkWidget *dock_object)
 {
     GdlDockObject *object;
 
-    g_return_if_fail (GDL_IS_DOCK_OBJECT (gtk_object));
+    g_return_if_fail (GDL_IS_DOCK_OBJECT (dock_object));
 
-    object = GDL_DOCK_OBJECT (gtk_object);
+    object = GDL_DOCK_OBJECT (dock_object);
     if (gdl_dock_object_is_compound (object)) {
         /* detach our dock object children if we have some, and even
            if we are not attached, so they can get notification */
@@ -305,7 +301,7 @@ gdl_dock_object_destroy (GtkObject *gtk_object)
     if (object->master)
         gdl_dock_object_unbind (object);
         
-    GDL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (gtk_object));
+    GTK_WIDGET_CLASS(gdl_dock_object_parent_class)->destroy (dock_object);
 }
 
 static void
@@ -326,7 +322,7 @@ gdl_dock_object_show (GtkWidget *widget)
                                (GtkCallback) gdl_dock_object_foreach_automatic,
                                gtk_widget_show);
     }
-    GDL_CALL_PARENT (GTK_WIDGET_CLASS, show, (widget));
+    GTK_WIDGET_CLASS (gdl_dock_object_parent_class)->show (widget);
 }
 
 static void
@@ -337,7 +333,7 @@ gdl_dock_object_hide (GtkWidget *widget)
                                (GtkCallback) gdl_dock_object_foreach_automatic,
                                gtk_widget_hide);
     }
-    GDL_CALL_PARENT (GTK_WIDGET_CLASS, hide, (widget));
+    GTK_WIDGET_CLASS (gdl_dock_object_parent_class)->hide (widget);
 }
 
 static void
@@ -360,8 +356,8 @@ gdl_dock_object_real_detach (GdlDockObject *object,
     GDL_DOCK_OBJECT_UNSET_FLAGS (object, GDL_DOCK_ATTACHED);
     parent = gdl_dock_object_get_parent_object (object);
     widget = GTK_WIDGET (object);
-    if (widget->parent)
-        gtk_container_remove (GTK_CONTAINER (widget->parent), widget);
+    if (gtk_widget_get_parent (widget))
+        gtk_container_remove (GTK_CONTAINER (gtk_widget_get_parent (GTK_WIDGET (widget))), widget);
     if (parent)
         gdl_dock_object_reduce (parent);
 }
@@ -495,9 +491,9 @@ gdl_dock_object_get_parent_object (GdlDockObject *object)
     
     g_return_val_if_fail (object != NULL, NULL);
 
-    parent = GTK_WIDGET (object)->parent;
+    parent = gtk_widget_get_parent (GTK_WIDGET (object));
     while (parent && !GDL_IS_DOCK_OBJECT (parent)) {
-        parent = parent->parent;
+        parent = gtk_widget_get_parent (parent);
     }
     
     return parent ? GDL_DOCK_OBJECT (parent) : NULL;
@@ -541,7 +537,8 @@ gdl_dock_object_reduce (GdlDockObject *object)
         return;
     }
 
-    GDL_CALL_VIRTUAL (object, GDL_DOCK_OBJECT_GET_CLASS, reduce, (object));
+    if (GDL_DOCK_OBJECT_GET_CLASS (object)->reduce)
+        GDL_DOCK_OBJECT_GET_CLASS (object)->reduce (object);
 }
 
 gboolean
@@ -551,12 +548,11 @@ gdl_dock_object_dock_request (GdlDockObject  *object,
                               GdlDockRequest *request)
 {
     g_return_val_if_fail (object != NULL && request != NULL, FALSE);
-    
-    return GDL_CALL_VIRTUAL_WITH_DEFAULT (object,
-                                          GDL_DOCK_OBJECT_GET_CLASS,
-                                          dock_request,
-                                          (object, x, y, request),
-                                          FALSE);
+
+    if (GDL_DOCK_OBJECT_GET_CLASS (object)->dock_request)
+        return GDL_DOCK_OBJECT_GET_CLASS (object)->dock_request (object, x, y, request);
+    else
+        return FALSE;
 }
 
 /**
@@ -677,11 +673,10 @@ gdl_dock_object_reorder (GdlDockObject    *object,
 {
     g_return_val_if_fail (object != NULL && child != NULL, FALSE);
 
-    return GDL_CALL_VIRTUAL_WITH_DEFAULT (object,
-                                          GDL_DOCK_OBJECT_GET_CLASS,
-                                          reorder,
-                                          (object, child, new_position, other_data),
-                                          FALSE);
+    if (GDL_DOCK_OBJECT_GET_CLASS (object)->reorder)
+            GDL_DOCK_OBJECT_GET_CLASS (object)->reorder (object, child, new_position, other_data);
+    else
+        return FALSE;
 }
 
 void 
@@ -697,7 +692,8 @@ gdl_dock_object_present (GdlDockObject *object,
         /* chain the call to our parent */
         gdl_dock_object_present (parent, object);
 
-    GDL_CALL_VIRTUAL (object, GDL_DOCK_OBJECT_GET_CLASS, present, (object, child));
+   if (GDL_DOCK_OBJECT_GET_CLASS (object)->present)
+        GDL_DOCK_OBJECT_GET_CLASS (object)->present (object, child);
 }
 
 /**
@@ -730,10 +726,10 @@ gdl_dock_object_child_placement (GdlDockObject    *object,
     if (!gdl_dock_object_is_compound (object))
         return FALSE;
     
-    return GDL_CALL_VIRTUAL_WITH_DEFAULT (object, GDL_DOCK_OBJECT_GET_CLASS,
-                                          child_placement,
-                                          (object, child, placement),
-                                          FALSE);
+    if (GDL_DOCK_OBJECT_GET_CLASS (object)->child_placement)
+        GDL_DOCK_OBJECT_GET_CLASS (object)->child_placement (object, child, placement);
+    else
+        return FALSE;
 }
 
 
